//
// HostIoReadingDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Localize;
using Ntp.Log;
using Ntp.Analyzer.Objects;
using Ntp.Data.Provider;

namespace Ntp.Analyzer.Data.Sql
{
    public sealed class HostIoReadingDatabaseMapper : FilteredSqlDatabaseMapper<HostIoReading>
    {
        internal HostIoReadingDatabaseMapper(HostDatabaseMapper hostMapper, LogBase log)
            : base(log)
        {
            this.hostMapper = hostMapper;
        }

        private const string SelectSql =
            "SELECT " +
            "id, time, hostId, timeSinceReset, " +
            "receiveBuffers, freeReceiveBuffers, usedReceiveBuffers, lowWaterRefills, " +
            "droppedPackets, ignoredPackets, receivedPackets, " +
            "packetsSent, packetsNotSent, interruptsHandled, receivedByInt " +
            "FROM hostIoReading ";

        private const string InsertSql =
            "INSERT INTO hostIoReading ( " +
            "time, hostId, timeSinceReset, " +
            "receiveBuffers, freeReceiveBuffers, usedReceiveBuffers, lowWaterRefills, " +
            "droppedPackets, ignoredPackets, receivedPackets, " +
            "packetsSent, packetsNotSent, interruptsHandled, receivedByInt) " +
            "VALUES ( " +
            "@time, @hostId, @timeSinceReset, " +
            "@receiveBuffers, @freeReceiveBuffers, @usedReceiveBuffers, @lowWaterRefills, " +
            "@droppedPackets, @ignoredPackets, @receivedPackets, " +
            "@packetsSent, @packetsNotSent, @interruptsHandled, @receivedByInt);{0};";

        private readonly HostDatabaseMapper hostMapper;

        protected override bool UseCache
        {
            get { return false; }
        }

        protected override string TableName
        {
            get { return "hostIoReading"; }
        }

        protected override string CreateSql
        {
            get
            {
                return
                    "CREATE TABLE hostIoReading ( " +
                    "    id {0} PRIMARY KEY, " +
                    "    time  TIMESTAMP NOT NULL, " +
                    "    hostId INT NOT NULL, " +
                    "    timeSinceReset INT NOT NULL, " +
                    "    receiveBuffers INT NOT NULL, " +
                    "    freeReceiveBuffers INT NOT NULL, " +
                    "    usedReceiveBuffers INT NOT NULL, " +
                    "    lowWaterRefills INT NOT NULL, " +
                    "    droppedPackets INT NOT NULL, " +
                    "    ignoredPackets INT NOT NULL, " +
                    "    receivedPackets INT NOT NULL, " +
                    "    packetsSent INT NOT NULL, " +
                    "    packetsNotSent INT NOT NULL, " +
                    "    interruptsHandled INT NOT NULL, " +
                    "    receivedByInt INT NOT NULL, " +
                    "    FOREIGN KEY (hostId) REFERENCES host(id) " +
                    "){1};";
            }
        }

        protected override void ReadContent()
        {
            throw new NotSupportedException(LogMessage.DatabaseCacheError);
        }

        protected override void Update(HostIoReading item)
        {
            throw new NotSupportedException(String.Format(LogMessage.DatabaseNoUpdate, TableName));
        }

        protected override void Insert(HostIoReading item)
        {
            lock (MapperLocker)
            {
                try
                {
                    Open();
                    Command.CommandText = SqlDatabaseFactory.Instance.PrepareInsertSql(InsertSql);
                    Command.Parameters.Add(CreateParameter("@time", item.Time));
                    Command.Parameters.Add(CreateParameter("@hostId", item.Host.Id));
                    Command.Parameters.Add(CreateParameter("@timeSinceReset", item.TimeSinceReset));
                    Command.Parameters.Add(CreateParameter("@receiveBuffers", item.ReceiveBuffers));
                    Command.Parameters.Add(CreateParameter("@freeReceiveBuffers", item.FreeReceiveBuffers));
                    Command.Parameters.Add(CreateParameter("@usedReceiveBuffers", item.UsedReceiveBuffers));
                    Command.Parameters.Add(CreateParameter("@lowWaterRefills", item.LowWaterRefills));
                    Command.Parameters.Add(CreateParameter("@droppedPackets", item.DroppedPackets));
                    Command.Parameters.Add(CreateParameter("@ignoredPackets", item.IgnoredPackets));
                    Command.Parameters.Add(CreateParameter("@receivedPackets", item.ReceivedPackets));
                    Command.Parameters.Add(CreateParameter("@packetsSent", item.PacketsSent));
                    Command.Parameters.Add(CreateParameter("@packetsNotSent", item.PacketsNotSent));
                    Command.Parameters.Add(CreateParameter("@interruptsHandled", item.InterruptsHandled));
                    Command.Parameters.Add(CreateParameter("@receivedByInt", item.ReceivedByInt));
                    var idObject = Command.ExecuteScalar();
                    item.SetId(Convert.ToInt32(idObject));
                }
                catch (Exception e)
                {
                    Log.WriteLine(String.Format(
                        LogMessage.DatabaseTableError, LogMessage.DatabaseInsert, TableName, e.Message),
                        Severity.Error);
                    Log.WriteLine(e, Severity.Trace);
                }
                finally
                {
                    Close();
                }
            }
        }

        /// <summary>
        /// Read all data from table in a sequential manner.
        /// </summary>
        /// <returns>The enumerator.</returns>
        public override IEnumerator<HostIoReading> GetEnumerator()
        {
            lock (MapperLocker)
            {
                bool error = false;
                string sql = PrepareSql(SelectSql);

                if (FilterId != null)
                {
                    sql += " WHERE Id = " + FilterId;
                }
                else
                {
                    if (FilterHost != null)
                        sql += " WHERE hostId = " + FilterHost.Id;

                    if (FilterTime != null)
                        sql += " AND time > '" + FilterTime.Value.ToString("yyyy-MM-dd HH:mm") + "'";
                }

                try
                {
                    Open();
                    Command.CommandText = sql;
                    Reader = Command.ExecuteReader();
                }
                catch (Exception e)
                {
                    Log.WriteLine(String.Format(
                        LogMessage.DatabaseTableError, LogMessage.DatabaseRead, TableName, e.Message),
                        Severity.Error);
                    Log.WriteLine(e, Severity.Trace);
                    error = true;
                }

                if (error)
                    yield break;

                while (Reader.Read())
                {
                    int id = Convert.ToInt32(Reader["id"]);
                    DateTime time = Convert.ToDateTime(Reader["time"]);
                    int hostId = Convert.ToInt32(Reader["hostId"]);
                    Host host = hostMapper[hostId];
                    int timeSinceReset = Convert.ToInt32(Reader["timeSinceReset"]);
                    int receiveBuffers = Convert.ToInt32(Reader["receiveBuffers"]);
                    int freeReceiveBuffers = Convert.ToInt32(Reader["freeReceiveBuffers"]);
                    int usedReceiveBuffers = Convert.ToInt32(Reader["usedReceiveBuffers"]);
                    int lowWaterRefills = Convert.ToInt32(Reader["lowWaterRefills"]);
                    int droppedPackets = Convert.ToInt32(Reader["droppedPackets"]);
                    int ignoredPackets = Convert.ToInt32(Reader["ignoredPackets"]);
                    int receivedPackets = Convert.ToInt32(Reader["receivedPackets"]);
                    int packetsSent = Convert.ToInt32(Reader["packetsSent"]);
                    int packetsNotSent = Convert.ToInt32(Reader["packetsNotSent"]);
                    int interruptsHandled = Convert.ToInt32(Reader["interruptsHandled"]);
                    int receivedByInt = Convert.ToInt32(Reader["receivedByInt"]);

                    var reading =
                        new HostIoReading(
                            id, time, host, timeSinceReset,
                            receiveBuffers, freeReceiveBuffers, usedReceiveBuffers, lowWaterRefills,
                            droppedPackets, ignoredPackets, receivedPackets,
                            packetsSent, packetsNotSent,
                            interruptsHandled, receivedByInt
                            );

                    yield return reading;
                }

                Close();
            }
        }
    }
}