//
// PeerGraphPageConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Config.Destination;
using Ntp.Analyzer.Config.Graph;
using Ntp.Analyzer.Localize;
using Ntp.Analyzer.Objects;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Page
{
    public sealed class PeerGraphPageConfiguration : GraphPageConfiguration
    {
        private PeerGraphPageConfiguration(
            HostConfiguration server,
            string configName,
            int indentLevel,
            int frequency,
            bool initialRun,
            bool fixedRun,
            string theme,
            string title,
            PeerPageConfiguration page,
            int linkIndex,
            DestinationCollection destinations,
            string link)
            : base(
                server,
                configName,
                indentLevel,
                frequency,
                initialRun,
                fixedRun,
                theme,
                title,
                linkIndex,
                destinations,
                link)
        {
            this.page = page;

            if (page != null)
            {
                page.AttachGraphPages(this);
            }
        }

        public const string Identifier = ConfigurationKeyword.PeerGraphPage;

        private new const bool DefaultInitialRun = true;
        private new const bool DefaultFixedRun = false;
        private const int DefaultFrequency = 0;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.PeerGraphPage,
            ConfigurationKeyword.Frequency,
            ConfigurationKeyword.InitialRun,
            ConfigurationKeyword.FixedRun,
            ConfigurationKeyword.PageTemplate,
            ConfigurationKeyword.PageTitle,
            ConfigurationKeyword.GraphPage,
            ConfigurationKeyword.LinkIndex,
            DirectoryCollection.Identifier,
            ConfigurationKeyword.PageLink
        };

        private readonly PeerPageConfiguration page;

        public override IEnumerable<GraphSetConfiguration> GraphSets
        {
            get { return page.Graphs; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                var items = new List<object> {ConfigName};

                if (ShowDefault || (!ShowDefault && Frequency != DefaultFrequency))
                    items.Add(Frequency);

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(FixedRun);

                if (ShowDefault || (!ShowDefault && Theme.ToLower() != DefaultTheme.ToLower()))
                    items.Add(Theme);

                items.Add(Title);

                if (page != null)
                    items.Add(page.ConfigName);

                if (ShowDefault || (!ShowDefault && LinkIndex != DefaultLinkIndex))
                    items.Add(LinkIndex);

                items.Add(Destinations);

                if (ShowDefault || (!ShowDefault && Link != DefaultLink))
                    items.Add(Link);

                return items.ToArray();
            }
        }

        public string GetFileName(Host host, string name, GraphSetConfiguration graphSet, GraphBaseConfiguration graph)
        {
            return GetGraphFileName(host, name, graphSet, graph);
        }

        public string GetLink(Host host, string name, GraphSetConfiguration graphSet, GraphBaseConfiguration graph)
        {
            // TODO: Sanitize link
            return WebPath + Link + GetGraphFileName(host, name, graphSet, graph);
        }

        private string GetGraphFileName(Host host, string name, GraphSetConfiguration graphSet,
            GraphBaseConfiguration graph)
        {
            return
                host.Name.Replace('.', '-') + "_" +
                graph.GetAltName(graphSet, name).
                    Replace(".png", String.Empty).
                    Replace('.', '-')
                + ".html";
        }

        public override string ToString()
        {
            var list = new List<string> {ConfigurationKeyword.PeerGraphPage};

            if (ShowDefault || (!ShowDefault && Frequency != DefaultFrequency))
                list.Add(ConfigurationKeyword.Frequency);

            if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                list.Add(ConfigurationKeyword.InitialRun);

            if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                list.Add(ConfigurationKeyword.FixedRun);

            if (ShowDefault || (!ShowDefault && Theme.ToLower() != DefaultTheme.ToLower()))
                list.Add(ConfigurationKeyword.PageTemplate);

            list.Add(ConfigurationKeyword.PageTitle);

            if (page != null)
                list.Add(ConfigurationKeyword.GraphPage);

            if (ShowDefault || (!ShowDefault && LinkIndex != DefaultLinkIndex))
                list.Add(ConfigurationKeyword.LinkIndex);

            list.Add(DirectoryCollection.Identifier);

            if (ShowDefault || (!ShowDefault && Link != DefaultLink))
                list.Add(ConfigurationKeyword.PageLink);

            return ConfigText(list.ToArray(), Items);
        }

        public static PeerGraphPageConfiguration Create(
            HostConfiguration server,
            ConfigurationBlock config,
            IEnumerable<PeerPageConfiguration> peerPages)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            string peerPageName = GetOptionString(options, ConfigurationKeyword.GraphPage, Identifier, true);
            PeerPageConfiguration peerPage = peerPages.SingleOrDefault(pp => pp.ConfigName == peerPageName);

            if (peerPage == null)
            {
                ConfigError(String.Format(
                    ConfigurationMessage.MissingReference,
                    ConfigurationKeyword.GraphPage, peerPageName, Identifier
                    ));

                ConfigError(ConfigurationMessage.ConfiguredGraphPages);
                foreach (PeerPageConfiguration peerPageConfig in peerPages)
                    ConfigError(peerPageConfig.ConfigName);
            }

            return new PeerGraphPageConfiguration(
                server,
                GetOptionConfigName(options, ConfigurationKeyword.PeerGraphPage, Identifier, true),
                config.IndentLevel,
                GetOptionInteger(options, ConfigurationKeyword.Frequency, Identifier, false, DefaultFrequency),
                GetOptionBoolean(options, ConfigurationKeyword.InitialRun, Identifier, false, DefaultInitialRun),
                GetOptionBoolean(options, ConfigurationKeyword.FixedRun, Identifier, false, DefaultFixedRun),
                GetOptionString(options, ConfigurationKeyword.PageTemplate, Identifier, false, DefaultTheme),
                GetOptionString(options, ConfigurationKeyword.PageTitle, Identifier, false, String.Empty),
                peerPage,
                GetOptionInteger(options, ConfigurationKeyword.LinkIndex, Identifier, false, DefaultLinkIndex),
                DirectoryCollection.Create(server, config),
                GetOptionString(options, ConfigurationKeyword.PageLink, Identifier, false)
                );
        }
    }
}