//
// StatsConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Config.Root;
using Ntp.Analyzer.Localize;

namespace Ntp.Analyzer.Config.Stats
{
    public abstract class StatsConfiguration : HostSubConfiguration
    {
        protected StatsConfiguration(
            string configName,
            int indentLevel,
            HostConfiguration server,
            ReadingBulkConfiguration bulk,
            DateTimeKind timeStamp,
            int frequency,
            bool initialRun,
            bool fixedRun
            )
            : base(server, configName, indentLevel, frequency, initialRun, fixedRun)
        {
            this.bulk = bulk;
            this.timeStamp = timeStamp;
        }

        protected const bool DefaultInitialRun = true;
        protected const bool DefaultFixedRun = true;
        private readonly ReadingBulkConfiguration bulk;
        private readonly DateTimeKind timeStamp;

        public ReadingBulkConfiguration Bulk
        {
            get { return bulk; }
        }

        public DateTimeKind TimeStamp
        {
            get { return bulk != null ? bulk.TimeStamp : timeStamp; }
        }

        public override bool InitialRun
        {
            get { return bulk != null ? bulk.InitialRun : base.InitialRun; }
        }

        public override bool FixedRun
        {
            get { return bulk != null ? bulk.FixedRun : base.FixedRun; }
        }

        public override int Frequency
        {
            get { return bulk != null ? bulk.Frequency : base.Frequency; }
        }

        protected abstract string InstanceName { get; }

        private String[] InstanceParams
        {
            get
            {
                var items = new List<string>
                {
                    InstanceName,
                    ConfigurationKeyword.Frequency
                };

                if (ShowDefault || (!ShowDefault && TimeStamp != DefaultTimeStamp))
                    items.Add(ConfigurationKeyword.TimeStamp);

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(ConfigurationKeyword.InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(ConfigurationKeyword.FixedRun);

                return items.ToArray();
            }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                var items = new List<object> {ConfigName, Frequency};

                if (ShowDefault || (!ShowDefault && TimeStamp != DefaultTimeStamp))
                    items.Add(GetTimezoneString(TimeStamp));

                if (ShowDefault || (!ShowDefault && InitialRun != DefaultInitialRun))
                    items.Add(InitialRun);

                if (ShowDefault || (!ShowDefault && FixedRun != DefaultFixedRun))
                    items.Add(FixedRun);

                return items.ToArray();
            }
        }

        public override string ToString()
        {
            return
                Bulk == null
                    ? ConfigText(InstanceParams, Items)
                    : ConfigText(
                        new[] {InstanceName, ConfigurationKeyword.Frequency},
                        new[] {ConfigName, Bulk.Name});
        }
    }
}