﻿//
// InterProcess.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using Ntp.Log;
#if __MonoCS__
using Mono.Unix;
using Mono.Unix.Native;
#else
using System.Threading;
#endif

namespace Ntp.Interop
{
    public static class InterProcess
    {
        public enum Signal
        {
            Error = 0,
            Exit,
            Reload,
            Refresh,
            Notify
        }

#if __MonoCS__

        private static UnixSignal sigterm, sigquit, sigint, sighup, sigusr1, sigusr2;
        private static UnixSignal[] signals = new UnixSignal[] {
            sigint = new UnixSignal (Signum.SIGINT),
            sigterm = new UnixSignal (Signum.SIGTERM),
            sigquit = new UnixSignal (Signum.SIGQUIT),
            sighup = new UnixSignal (Signum.SIGHUP),
            sigusr1 = new UnixSignal (Signum.SIGUSR1),
            sigusr2 = new UnixSignal (Signum.SIGUSR2)
        };

        public static Signal Wait (string name, LogBase log)
        {
            int i = 0;
            while (true) {
                try {
                    i = UnixSignal.WaitAny (signals, -1);
                } catch (Exception e) {
                    log.WriteLine ("Unrecoverable error in UnixSignal.WaitAny()",
                        Severity.Error);
                    log.WriteLine (e, Severity.Trace);
                    return Signal.Error;
                }

                if (i < 0 || i >= signals.Length) {
                    log.WriteLine (String.Format (
                        "Received unknown signal {0}",
                        signals [i].Signum.ToString ()),
                        Severity.Warn);
                    continue;
                }

                log.WriteLine (String.Format (
                    "Received signal {0}",
                    signals [i].Signum.ToString ()),
                    Severity.Debug);

                if (sigint.IsSet || sigterm.IsSet || sigquit.IsSet) {
                    sigint.Reset ();
                    sigterm.Reset ();
                    sigquit.Reset ();
                    return Signal.Exit;
                } else if (sighup.IsSet) {
                    sighup.Reset ();
                    return Signal.Refresh;
                } else if (sigusr1.IsSet) {
                    sigusr1.Reset ();
                    return Signal.Reload;
                } else if (sigusr2.IsSet) {
                    sigusr2.Reset ();
                    return Signal.Notify;
                }
            }
        }

        #else

        public static bool Release(string name, LogBase log)
        {
            try
            {
                EventWaitHandle handle = EventWaitHandle.OpenExisting(name);
                handle.Set();
                return true;
            }
            catch (Exception e)
            {
                log.WriteLine("Error in mutex handling.", Severity.Error);
                log.WriteLine(e, Severity.Trace);
                return false;
            }
        }

        public static Signal Wait(string name, LogBase log)
        {
            try
            {
                var handle = new EventWaitHandle(
                    false, EventResetMode.ManualReset,
                    name);
                handle.WaitOne();
                return Signal.Exit;
            }
            catch (Exception e)
            {
                log.WriteLine("Error in mutex handling.", Severity.Error);
                log.WriteLine(e, Severity.Trace);
                return Signal.Error;
            }
        }

#endif
    }
}